"
    Copyright (c) 2018 Jan Vrany
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, 
    are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, 
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice, 
       this list of conditions and the following disclaimer in the documentation 
       and/or other materials provided with the distribution.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS' AND 
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
    ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
    ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"
"{ Package: 'jv:dragonfly/udis86sx' }"

"{ NameSpace: Smalltalk }"

UninterpretedBytes variableByteSubclass:#UDIS86Operand
	instanceVariableNames:''
	classVariableNames:'Size Offset_type Offset_size Offset_base Offset_index
		Offset_scale Offset_offset Offset_lval'
	poolDictionaries:''
	category:'UDIS86'
!

!UDIS86Operand primitiveDefinitions!
%{

/*
 * includes, defines, structure definitions
 * and typedefs come here.
 */

#include <stddef.h>
#include <udis86.h>

%}
! !

!UDIS86Operand class methodsFor:'documentation'!

copyright
"
    Copyright (c) 2018 Jan Vrany
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, 
    are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, 
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice, 
       this list of conditions and the following disclaimer in the documentation 
       and/or other materials provided with the distribution.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS' AND 
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
    ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
    ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"
!

documentation
"
    UDIS86Operand represents a single operand of an instruction. 
    It contains the following fields.

        size
        type
        base
        index
        scale
        offset
        lval

    unsigned ud_operand_t.size

        Size of the operand in number of bits.

    enum ud_operand_type ud_operand_t.type

        Type of the operand. Possible values are,

        UD_OP_MEM

            A memory operand. The intermediate form normalizes all memory address equations to the scale-index-base form. The address equation is available in,

                base - base register as an enumerated constant of type enum ud_type. Maybe UD_NONE, in which case the memory addressing form does not include a base register.
                index - index register as an enumerated constant of type enum ud_type. Maybe UD_NONE, in which case the memory addressing form does not include an index register.
                scale - an integer value by which the index register must be scaled. Maybe 0, denoting the absence of a scale component in the address.
                offset - An integer value, which if non-zero represents the size of the displacement offset, and is one of 8, 16, 32, and 64. The value is available in lval.

        UD_OP_PTR

            A segment:offset pointer operand. The size field can have two values, 32 (for 16:16 seg:off) and 48 (for 16:32 seg:off). The pointer value is available in lval (as lval.ptr.seg and lval.ptr.off)

        UD_OP_IMM

            An Immediate operand. Value available in lval.

        UD_OP_JIMM

            An Immediate operand to a branch instruction (relative offsets). Value available in lval.

        UD_OP_CONST

            Implicit constant operand. Value available in lval.

        UD_OP_REG

            A register operand. The specific register is available in the base field as an enumerated constant of type enum ud_type.

    enum ud_register ud_operand_t.base

        Contains an enumerated constant of type enum ud_type representing a register operand or the base of a memory operand.

    enum ud_register ud_operand_t.index

        Contains an enumerated constant of type enum ud_type representing the index register of a memory operand.

    unsigned ud_operand_t.scale

        Contains the scale component of a memory address operand.

    unsigned ud_operand_t.offset

        Contains the size of the displacement component of a memory address operand. The displacement itself is given by lval.

    ud_lval_t ud_operand_t.lval

        A union data structure that aggregates integer fields of different sizes, storing values depending on the type and size of the operand.

        lval.sbyte

            Signed Byte

        lval.ubyte

            Unsigned Byte

        lval.sword

            Signed Word

        lval.uword

            Unsigned Word

        lval.sdword

            Signed Double Word

        lval.udword

            Unsigned Double Word

        lval.sqword

            Signed Quad Word

        lval.uqword

            Unsigned Quad Word


    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]
        http://udis86.sourceforge.net/manual/libudis86.html#inspect-operands

"
! !

!UDIS86Operand class methodsFor:'initialization'!

initialize
    "Invoked at system start or when the class is dynamically loaded."

    | v |

    super initialize.
%{
    v = __MKSMALLINT ( sizeof ( ud_operand_t ) );
%}.
    Size := v.

%{
    v = __MKSMALLINT ( offsetof ( ud_operand_t, type ) );
%}.
    Offset_type := v.
%{
    v = __MKSMALLINT ( offsetof ( ud_operand_t, size ) );
%}.
    Offset_size := v.
%{
    v = __MKSMALLINT ( offsetof ( ud_operand_t, base ) );
%}.
    Offset_base := v.
%{
    v = __MKSMALLINT ( offsetof ( ud_operand_t, index ) );
%}.
    Offset_index := v.
%{
    v = __MKSMALLINT ( offsetof ( ud_operand_t, scale ) );
%}.
    Offset_scale := v.
%{
    v = __MKSMALLINT ( offsetof ( ud_operand_t, lval ) );
%}.
    Offset_lval := v.

! !

!UDIS86Operand class methodsFor:'instance creation'!

new
    ^ self basicNew: Size

    "Created: / 25-06-2018 / 16:14:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

new: anInteger
    self shouldNotImplement

    "Created: / 25-06-2018 / 16:14:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!UDIS86Operand methodsFor:'accessing'!

base
     ^ self signedInt32At: Offset_base + 1

    "Created: / 25-06-2018 / 15:41:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-06-2018 / 10:12:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

index
     ^ self signedInt32At: Offset_index + 1

    "Created: / 25-06-2018 / 15:41:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-06-2018 / 10:13:06 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

offset
     ^ self byteAt: Offset_offset + 1

    "Created: / 25-06-2018 / 15:41:41 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-06-2018 / 10:13:44 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

sbyte
     ^ self signedByteAt: Offset_lval + 1

    "Created: / 26-06-2018 / 10:15:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

scale
     ^ self byteAt: Offset_scale + 1

    "Created: / 25-06-2018 / 15:41:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-06-2018 / 10:13:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

sdword
     ^ self signedInt32At: Offset_lval + 1

    "Created: / 26-06-2018 / 10:16:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

size
    ^ self byteAt: Offset_size + 1

    "Created: / 25-06-2018 / 15:41:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-06-2018 / 10:12:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

sqword
     ^ self signedInt64At: Offset_lval + 1

    "Created: / 26-06-2018 / 10:16:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

sval
    | size |

    size := self size.
    size ==  8 ifTrue:[ ^ self sbyte ].
    size == 16 ifTrue:[ ^ self sword ].
    size == 32 ifTrue:[ ^ self sdword ].
    size == 64 ifTrue:[ ^ self sqword ].
    self error: 'Invalid operand size!!'.

    "Created: / 26-06-2018 / 10:31:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

sword
     ^ self signedInt16At: Offset_lval + 1

    "Created: / 26-06-2018 / 10:16:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

type
    ^ self signedInt32At: Offset_type + 1

    "Created: / 25-06-2018 / 15:41:21 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-06-2018 / 10:11:27 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

ubyte
     ^ self byteAt: Offset_lval + 1

    "Created: / 26-06-2018 / 10:15:50 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

udword
     ^ self unsignedInt32At: Offset_lval + 1

    "Created: / 26-06-2018 / 10:16:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

uqword
     ^ self unsignedInt64At: Offset_lval + 1

    "Created: / 26-06-2018 / 10:16:41 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

uval
    | size |

    size := self size.
    size ==  8 ifTrue:[ ^ self ubyte ].
    size == 16 ifTrue:[ ^ self uword ].
    size == 32 ifTrue:[ ^ self udword ].
    size == 64 ifTrue:[ ^ self uqword ].
    self error: 'Invalid operand size!!'.

    "Created: / 26-06-2018 / 10:31:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

uword
     ^ self unsignedInt16At: Offset_lval + 1

    "Created: / 26-06-2018 / 10:16:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!UDIS86Operand class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !


UDIS86Operand initialize!
